#include "SoftwareSerial.h"
#include "DFRobotDFPlayerMini.h"

// Define connections for SoftwareSerial
SoftwareSerial mySerial(10, 11); // TX = 10, RX = 11
DFRobotDFPlayerMini myDFPlayer;

// Button Setup
const int buttonPin = 2;
int buttonState;
int lastButtonState = HIGH;
bool isPlaying = false;
int currentTrack = 1;
int totalTracks = 0;

// Volume Button Setup
const int volUpButton = 3;
const int volDownButton = 4;
int volUpState;
int volDownState;
int volume = 20;
int lastVolume = -1;

void setup() {
  // Setup Buttons
  pinMode(buttonPin, INPUT_PULLUP);    // Play/Next Button
  pinMode(volUpButton, INPUT_PULLUP);  // Volume Up Button
  pinMode(volDownButton, INPUT_PULLUP);// Volume Down Button

  // Start communication with DFPlayer Mini
  mySerial.begin(9600);
  Serial.begin(9600);

  // Initialize DFPlayer
  if (!myDFPlayer.begin(mySerial)) {
    Serial.println("DFPlayer Mini not detected!");
    while (true);
  }
  Serial.println("DFPlayer Mini ready!");

  myDFPlayer.setTimeOut(500);

  // Get total number of tracks on SD card
  totalTracks = myDFPlayer.readFileCounts();
  Serial.print("Total Tracks: ");
  Serial.println(totalTracks);

  // Set initial volume
  myDFPlayer.volume(volume);
}

void loop() {
  // Check Play/Next Button Press
  buttonState = digitalRead(buttonPin);

  if (buttonState == LOW && lastButtonState == HIGH) {
    delay(50); // Debounce delay
    if (isPlaying) {
      currentTrack++; // Next track
      
      // Cycle back to track 1 if at the end
      if (currentTrack > totalTracks) {
        currentTrack = 1;
      }

      myDFPlayer.play(currentTrack);
      Serial.print("Playing track: ");
      Serial.println(currentTrack);
    } else {
      myDFPlayer.play(currentTrack); // Start first track
      isPlaying = true;
      Serial.print("Playing track: ");
      Serial.println(currentTrack);
    }
  }
  lastButtonState = buttonState;

  // Check Volume Up Button
  volUpState = digitalRead(volUpButton);
  if (volUpState == LOW) {
    delay(50); // Debounce delay
    volume++;
    if (volume > 30) volume = 30; // Cap volume to max 30
    myDFPlayer.volume(volume);
    Serial.print("Volume Up: ");
    Serial.println(volume);
    delay(200); // Short delay to avoid rapid changes
  }

  // Check Volume Down Button
  volDownState = digitalRead(volDownButton);
  if (volDownState == LOW) {
    delay(50); // Debounce delay
    volume--;
    if (volume < 0) volume = 0; // Cap volume to min 0
    myDFPlayer.volume(volume);
    Serial.print("Volume Down: ");
    Serial.println(volume);
    delay(200); // Short delay to avoid rapid changes
  }
}




